@@ -6,16 +6,16 @@
 ## respectively the data.frame containing the index and the dimensions
 ## of the panel
 
-## pdata.frame : 
+## pdata.frame:
 ## - $<-
 ## - [
-## - $    
+## - $
 ## - [[
 ## - print
 ## - as.list
 ## - as.data.frame
 
-## pseries :
+## pseries:
 ## - print
 ## - as.matrix
 ## - plot
@@ -24,7 +24,7 @@
 ## - print.summary
 ## - is.pseries
 
-## pdim :
+## pdim:
 ## - pdim.default
 ## - pdim.data.frame
 ## - pdim.pdata.frame
@@ -33,16 +33,16 @@
 ## - pdim.pgmm
 ## - print.pdim
  
-## index :
-## - index.pindex    
+## index:
+## - index.pindex
 ## - index.pdata.frame
 ## - index.pseries
 ## - index.panelmodel
 
 fancy.row.names <- function(index, sep = "-") {
-  if (length(index) == 2) {result <- paste(index[[1]], index[[2]], sep = sep)}
+  if (length(index) == 2) {result <- paste(index[[1L]], index[[2L]], sep = sep)}
   # this in the order also used for sorting (group, id, time):
-  if (length(index) == 3) {result <- paste(index[[3]], index[[1]], index[[2]], sep = sep)}
+  if (length(index) == 3) {result <- paste(index[[3L]], index[[1L]], index[[2L]], sep = sep)}
   return(result)
 }
 
@@ -55,19 +55,22 @@ fancy.row.names <- function(index, sep = "-") {
 #' attribute that describes its individual and time dimensions.
 #' 
 #' The `index` argument indicates the dimensions of the panel. It can
-#' be: \itemize{ \item a vector of two character strings which
-#' contains the names of the individual and of the time indexes, \item
+#' be: \itemize{
+#' \item a vector of two character strings which
+#' contains the names of the individual and of the time indexes,
+#' \item
 #' a character string which is the name of the individual index
 #' variable. In this case, the time index is created automatically and
 #' a new variable called "time" is added, assuming consecutive and
-#' ascending time periods in the order of the original data, \item an
-#' integer, the number of individuals. In this case, the data need to
-#' be a balanced panel and be organized as a stacked time series
+#' ascending time periods in the order of the original data,
+#' \item an integer, the number of individuals. In this case, the data
+#' need to be a balanced panel and be organized as a stacked time series
 #' (successive blocks of individuals, each block being a time series
 #' for the respective individual) assuming consecutive and ascending
 #' time periods in the order of the original data. Two new variables
 #' are added: "id" and "time" which contain the individual and the
-#' time indexes.  }
+#' time indexes.
+#' }
 #' 
 #' The `"[["` and `"$"` extract a series from the `pdata.frame`.  The
 #' `"index"` attribute is then added to the series and a class
@@ -76,8 +79,11 @@ fancy.row.names <- function(index, sep = "-") {
 #' `index` attribute.  A safe way to extract the index attribute is to
 #' use the function [index()] for 'pdata.frames' (and other objects).
 #' 
-#' `as.data.frame` removes the index from the `pdata.frame` and adds
-#' it to each column.
+#' `as.data.frame` removes the index attribute from the `pdata.frame`
+#' and adds it to each column. For its argument `row.names` set to 
+#' `FALSE` row names are an integer series, `TRUE` gives "fancy" row
+#' names; if a character (with length of the resulting data frame),
+#' the row names will be the character's elements.
 #' 
 #' `as.list` behaves by default identical to
 #' [base::as.list.data.frame()] which means it drops the
@@ -97,10 +103,11 @@ fancy.row.names <- function(index, sep = "-") {
 #' @param drop.index logical, indicates whether the indexes are to be
 #'     excluded from the resulting pdata.frame,
 #' @param optional see [as.data.frame()],
-#' @param row.names `NULL` or logical, indicates whether ``fancy'' row
-#'     names (a combination of individual index and time index) are to
+#' @param row.names `NULL` or logical, indicates whether "fancy" row
+#'     names (combination of individual index and time index) are to
 #'     be added to the returned (p)data.frame (`NULL` and `FALSE` have
-#'     the same meaning),
+#'     the same meaning for `pdata.frame`; for
+#'     `as.data.frame.pdata.frame` see Details),
 #' @param stringsAsFactors logical, indicating whether character
 #'     vectors are to be converted to factors,
 #' @param replace.non.finite logical, indicating whether values for
@@ -170,6 +177,11 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
         stop("'index' can be of length 3 at the most (one index variable for individual, time, group)")
     }
     
+    # prune input: x is supposed to be a plain data.frame. Other classes building
+    # on top of R's data frame can inject attributes etc. that confuse functions
+    # in pkg plm.
+    x <- data.frame(x)
+    
     # if requested: coerce character vectors to factors
     if (stringsAsFactors) {
         x.char <- names(x)[sapply(x, is.character)]
@@ -229,11 +241,11 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
       x <- x[, !cst.check]
     }
   
-    # sanity check for 'index' argument. First check the presence of a
+    # sanity check for 'index' argument. First, check the presence of a
     # grouping variable, this should be the third element of the index
     # vector or any "group" named element of this vector
     group.name <- NULL
-    if (! is.null(names(index)) | length(index == 3)){
+    if (! is.null(names(index)) || length(index == 3)){
         if (! is.null(names(index))){
             grouppos <- match("group", names(index))
             if (! is.na(grouppos)){
@@ -261,14 +273,14 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
     # if the length of index is 2, the first element is id, the second
     # is time
     if (length(index) == 2){
-        id <- index[1]
-        time <- index[2]
+        id <- index[1L]
+        time <- index[2L]
     }
     # if both id and time are NULL, the names of the index are the first
     # two names of x
-    if (is.null(id) & is.null(time)){
-        id.name <- names(x)[1]
-        time.name <- names(x)[2]
+    if (is.null(id) && is.null(time)){
+        id.name <- names(x)[1L]
+        time.name <- names(x)[2L]
     }
     else{
         id.name <- id
@@ -299,8 +311,8 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
         }
     }
     else{
-        # id.name is not numeric, i.e. individual index is supplied
-        if (!id.name %in% names(x)) stop(paste("variable ",id.name," does not exist (individual index)", sep=""))
+        # id.name is not numeric, i.e., individual index is supplied
+        if (!id.name %in% names(x)) stop(paste("variable ", id.name, " does not exist (individual index)", sep=""))
         
         if (is.factor(x[[id.name]])){
             id <- x[[id.name]] <- x[[id.name]][drop=TRUE] # drops unused levels of factor
@@ -313,7 +325,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
             # if no time index is supplied, add time variable
             # automatically order data by individual index, necessary
             # for the automatic addition of time index to be
-            # succesfull if no time index was supplied
+            # successful if no time index was supplied
             x <- x[order(x[[id.name]]), ]
             Ti <- table(x[[id.name]]) # was: Ti <- table(id)
             n <- length(Ti)
@@ -372,17 +384,30 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
     index <- x[, posindex]
     if (drop.index) {
         x <- x[ , -posindex, drop = FALSE]
-        if (ncol(x) == 0L) cat("after dropping of index variables, the pdata.frame contains 0 columns")
+        if (ncol(x) == 0L) warning("after dropping of index variables, the pdata.frame contains 0 columns")
     }
     
-    test_doub <- table(index[[1]], index[[2]], useNA = "ifany")
-    if (anyNA(colnames(test_doub)) || anyNA(rownames(test_doub)))
-        cat(paste0("at least one couple (id-time) has NA in at least one index dimension",
-                   "in resulting pdata.frame\n to find out which, use e.g.",
-                   "table(index(your_pdataframe), useNA = \"ifany\")\n"))
-    if (any(as.vector(test_doub[!is.na(rownames(test_doub)), !is.na(colnames(test_doub))]) > 1))
-        warning(paste("duplicate couples (id-time) in resulting pdata.frame\n to find out which,",
-                      "use e.g. table(index(your_pdataframe), useNA = \"ifany\")"))
+    ### warn if duplicate couples
+    test_doub <- table(index[[1L]], index[[2L]], useNA = "ifany")
+    if (any(as.vector(test_doub[!is.na(rownames(test_doub)), !is.na(colnames(test_doub))]) > 1L))
+      warning(paste("duplicate couples (id-time) in resulting pdata.frame\n to find out which,",
+                    "use, e.g., table(index(your_pdataframe), useNA = \"ifany\")"))
+    
+    ### warn if NAs in index as likely not sane
+    if (anyNA(index[[1L]]) || anyNA(index[[2L]]) || (if(ncol(index) == 3L) anyNA(index[[3L]]) else FALSE))
+        warning(paste0("at least one NA in at least one index dimension ",
+                       "in resulting pdata.frame\n to find out which, use, e.g., ",
+                       "table(index(your_pdataframe), useNA = \"ifany\")\n"))
+    
+    ### Could also remove rows with NA in any index' dimension
+    # drop.rows <- is.na(index[[1L]]) | is.na(index[[2L]])
+    # if(ncol(index) == 3L) drop.rows <- drop.rows | is.na(index[[3L]])
+    # if((no.drop.rows <- sum(drop.rows)) > 0L) {
+    #   x <- x[!drop.rows, ]
+    #   index <- index[!drop.rows, ]
+    #   txt.drop.rows <- paste0(no.drop.rows, " row(s) dropped in resulting pdata.frame due to NA(s) in at least one index dimension")
+    #   warning(txt.drop.rows)
+    # }
     
     if (row.names) {
         attr(x, "row.names") <- fancy.row.names(index)
@@ -401,7 +426,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
 
 #' @rdname pdata.frame
 #' @export
-"$<-.pdata.frame" <- function(x, name, value){
+"$<-.pdata.frame" <- function(x, name, value) {
   if (inherits(value, "pseries")){
     # remove pseries features before adding value as a column to pdata.frame
     if (length(class(value)) == 1) value <- unclass(value)
@@ -457,6 +482,35 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
 #   return(result)
 # }
 
+## Non-exported internal function for subsetting of pseries. Can be used
+## internally. 
+## Currently we do not have "proper" subsetting function for pseries
+## ([.pseries) which applies in any case (but see the sketch to be tested in
+##  tool_pdata.frame.R)
+subset_pseries <- function(x, ...) {
+  
+  ## use '...' instead of only one specific argument, because subsetting for
+  ## factors can have argument 'drop', e.g., x[i, drop=TRUE] see ?Extract.factor
+  index <- attr(x, "index")
+  if (is.null(index)) warning("pseries object with is.null(index(pseries)) == TRUE encountered, trying to continue anyway...")
+  if (!is.index(index)) stop(paste0("pseries object has illegal index with class(index) == ", paste0(class(index), collapse = ", ")))
+  names_orig <- names(x)
+  x <- remove_pseries_features(x)
+  result <- x[...]
+  
+  # subset index / identify rows to keep in the index:
+  keep_rownr <- seq_along(names_orig)  # full length row numbers original pseries
+  names(keep_rownr) <- names_orig
+  keep_rownr <- keep_rownr[names(result)] # row numbers to keep after subsetting
+  index <- index[keep_rownr, ]
+  
+  # drop unused levels (like in subsetting of pdata.frames)
+  index <- droplevels(index)
+  
+  result <- add_pseries_features(result, index)
+  return(result)
+}
+
 
 #' @rdname pdata.frame
 #' @export
@@ -517,12 +571,12 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
     }
     
     # delete attribute with old index first:
-    # this preseves the order of the attributes because 
+    # this preserves the order of the attributes because 
     # order of non-standard attributes is scrambled by R's data.frame subsetting with `[.`
     # (need to add new index later anyway)
     attr(x, "index") <- NULL
     
-    # Set class to "data.frame" first to avoid coering which enlarges the (p)data.frame 
+    # Set class to "data.frame" first to avoid coercing which enlarges the (p)data.frame 
     # (probably by as.data.frame.pdata.frame).
     # Coercing is the built-in behaviour for extraction from data.frames by "[." (see ?`[.data.frame`) 
     # and it seems this cannot be avoided; thus we need to make sure, not to have any coercing going on
@@ -545,7 +599,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
     
     mydata <- eval(sc_mod)
 
-    if (is.null(dim(mydata))){
+    if (is.null(dim(mydata))) {
       # subsetting returned a vector or a factor or NULL (nothing more is left)
       if (is.null(mydata)) {
         # since R 3.4.0 NULL cannot have attributes, so special case it
@@ -569,7 +623,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
 
 #' @rdname pdata.frame
 #' @export
-"[[.pdata.frame" <- function(x, y){
+"[[.pdata.frame" <- function(x, y) {
   index <- attr(x, "index")
   attr(x, "index") <- NULL
   class(x) <- "data.frame"
@@ -588,13 +642,13 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
 
 #' @rdname pdata.frame
 #' @export
-"$.pdata.frame" <- function(x, y){
+"$.pdata.frame" <- function(x, y) {
     "[[.pdata.frame"(x, paste(as.name(y)))
 }
 
 #' @rdname pdata.frame
 #' @export
-print.pdata.frame <- function(x, ...){
+print.pdata.frame <- function(x, ...) {
   attr(x, "index") <- NULL
   class(x) <- "data.frame"
   # This is a workaround: print.data.frame cannot handle
@@ -618,7 +672,7 @@ print.pdata.frame <- function(x, ...){
 #
 #  By setting argument keep.attributes = TRUE, the attributes of the pdata.frame
 #  are preserved by as.list.pdata.frame: a list of pseries is returned
-#  and lapply can be used as usual, now working on a list of pseries, e.g.
+#  and lapply can be used as usual, now working on a list of pseries, e.g.,
 #    lapply(as.list(pdata.frame[ , your_cols], keep.attributes), lag)
 #  works as expected.
 
@@ -648,38 +702,49 @@ as.list.pdata.frame <- function(x, keep.attributes = FALSE, ...) {
 
 #' @rdname pdata.frame
 #' @export
-as.data.frame.pdata.frame <- function(x, row.names = NULL, optional = FALSE, keep.attributes = TRUE, ...){
+as.data.frame.pdata.frame <- function(x, row.names = NULL, optional = FALSE, keep.attributes = TRUE, ...) {
     index <- attr(x, "index")
 
-     if (!keep.attributes) {
-       attr(x, "index") <- NULL
-       class(x) <- "data.frame"
-       rownames(x) <- NULL
-     } else {
-      # make each column a pseries
+    if(!keep.attributes) {
+      attr(x, "index") <- NULL
+      class(x) <- "data.frame"
+      rownames(x) <- NULL
+    } else {
+      # make each column a pseries (w/o names)
       x <- lapply(x,
                   function(z){
-                  #     names(z) <- row.names(x) # it does not seem possible to keep the names in the 'pseries' because the call to data.frame later deletes the names 
+                  #     names(z) <- row.names(x) # it is not possible to keep the names in the 'pseries'/
+                                                 # in columns because the call to data.frame later deletes
+                                                 # the names attribute of columns (definition of data frame)
                     attr(z, "index") <- index
                     class(z) <- base::union("pseries", class(z))
                     return(z)
-                  }
-      )
+                  })
     }
     
-    if (is.null(row.names) || row.names == FALSE) {
-        x <- data.frame(x)
+    if(is.null(row.names)) {
+      # do as base::as.data.frame does for NULL
+      x <- as.data.frame(x, row.names = NULL)
     } else {
-        if (row.names == TRUE) { # set fancy row names
-            x <- data.frame(x)
-            row.names(x) <- fancy.row.names(index)
-            # using row.names(x) <- "something" is safer (does not allow
-            # duplicate row.names) than # attr(x,"row.names") <- "something"
+      if(is.logical(row.names) && row.names == FALSE) {
+        # set row names to integer sequence 1, 2, 3, ...
+        x <- as.data.frame(x)
+        row.names(x) <- NULL
+      }
+      if(is.logical(row.names) && row.names == TRUE) {
+        # set fancy row names
+        x <- as.data.frame(x)
+        row.names(x) <- fancy.row.names(index)
+      }
+      if(is.character(row.names)) {
+        x <- as.data.frame(x)
+        row.names(x) <- row.names
+      }
+      if(!(isTRUE(row.names) || isFALSE(row.names) || is.character(row.names)))
+        stop("argument 'row.names' is none of NULL, FALSE, TRUE, and not a character")
+      # using row.names(x) <- "something" is safer (does not allow
+      # duplicate row.names) than # attr(x,"row.names") <- "something"
     }
-    ## not implemented: if row.names is a character vector, row.names
-    ## could also be passed here to base::data.frame, see
-    ## ?base::data.frame
-    } 
     return(x)
 }
 
@@ -739,7 +804,7 @@ is.pseries <- function(object) {
   if (!inherits(object, "pseries")) res <- FALSE
   # class 'pseries' is always on top of basic class: min 2 classes needed, if 2 classes "pseries" needs to be first entry
   if (!length(class(object)) >= 2L) res <- FALSE
-  if (length(class(object)) == 2L & class(object)[1] != "pseries") res <- FALSE
+  if (length(class(object)) == 2L && class(object)[1] != "pseries") res <- FALSE
   if (!has.index(object)) res <- FALSE
   if (!any(c(is.numeric(object), is.factor(object), is.logical(object), 
              is.character(object), is.complex(object)))) {
@@ -823,13 +888,13 @@ is.pseries <- function(object) {
 #'          model="random", subset = gsp > 5000)
 #' pdim(z)
 #' 
-pdim <- function(x, ...){
+pdim <- function(x, ...) {
   UseMethod("pdim")
 }
 
 #' @rdname pdim
 #' @export
-pdim.default <- function(x, y, ...){
+pdim.default <- function(x, y, ...) {
   if (length(x) != length(y)) stop("The length of the two vectors differs\n")
   x <- x[drop = TRUE] # drop unused factor levels so that table() 
   y <- y[drop = TRUE] # gives only needed combinations
@@ -856,56 +921,56 @@ pdim.default <- function(x, y, ...){
 
 #' @rdname pdim
 #' @export
-pdim.data.frame <- function(x, index = NULL, ...){
+pdim.data.frame <- function(x, index = NULL, ...) {
   x <- pdata.frame(x, index)
   index <- attr(x, "index")
-  id <- index[[1]]
-  time <- index[[2]]
+  id <- index[[1L]]
+  time <- index[[2L]]
   pdim(id,time)
 }
 
 #' @rdname pdim
 #' @export
-pdim.pdata.frame <- function(x,...){
+pdim.pdata.frame <- function(x,...) {
   index <- attr(x, "index")
-  pdim(index[[1]],index[[2]])
+  pdim(index[[1L]], index[[2L]])
 }
 
 #' @rdname pdim
 #' @export
 pdim.pseries <- function(x,...) {
   index <- attr(x, "index")
-  pdim(index[[1]], index[[2]])
+  pdim(index[[1L]], index[[2L]])
 }
 
 #' @rdname pdim
 #' @export
-pdim.panelmodel <- function(x, ...){
+pdim.panelmodel <- function(x, ...) {
   x <- model.frame(x)
   pdim(x)
 }
 
 #' @rdname pdim
 #' @export
-pdim.pgmm <- function(x, ...){
+pdim.pgmm <- function(x, ...) {
 ## pgmm is also class panelmodel, but take advantage of the pdim attribute in it
   attr(x, "pdim")
 }
 
 #' @rdname pdim
 #' @export
-print.pdim <- function(x, ...){
+print.pdim <- function(x, ...) {
   if (x$balanced){
       cat("Balanced Panel: ")
-      cat(paste("n = ",x$nT$n,", ",sep=""))
-      cat(paste("T = ",x$nT$T,", ",sep=""))
-      cat(paste("N = ",x$nT$N,"\n",sep=""))
+      cat(paste("n = ", x$nT$n, ", ", sep=""))
+      cat(paste("T = ", x$nT$T, ", ", sep=""))
+      cat(paste("N = ", x$nT$N, "\n", sep=""))
   }
   else{
       cat("Unbalanced Panel: ")
-      cat(paste("n = ",x$nT$n,", ",sep=""))
-      cat(paste("T = ",min(x$Tint$Ti),"-",max(x$Tint$Ti),", ",sep=""))
-      cat(paste("N = ",x$nT$N,"\n",sep=""))
+      cat(paste("n = ", x$nT$n,", ", sep=""))
+      cat(paste("T = ", min(x$Tint$Ti), "-", max(x$Tint$Ti), ", ", sep=""))
+      cat(paste("N = ", x$nT$N, "\n", sep=""))
   }
 }
 
@@ -933,6 +998,9 @@ print.pdim <- function(x, ...){
 #' frame was constructed with such).)
 #' 
 #' @name index.plm
+#' @aliases index
+#' @importFrom zoo index 
+#' @export index
 #' @param x an object of class `"pindex"`, `"pdata.frame"`,
 #'     `"pseries"` or `"panelmodel"`,
 #' @param which the index(es) to be extracted (see details),
@@ -959,12 +1027,13 @@ print.pdim <- function(x, ...){
 #' index(pProduc, 3)
 #' index(pProduc, "region")
 #' index(pProduc, "group")
-#' 
+#'
 NULL
 
 #' @rdname index.plm
 #' @export
-index.pindex <- function(x, which = NULL, ...){
+index.pindex <- function(x, which = NULL, ...) {
+
     if (is.null(which)) {
       # if no specific index is requested, select all index variables
       which <- names(x)
@@ -979,37 +1048,105 @@ index.pindex <- function(x, which = NULL, ...){
     if (is.numeric(which)){
         if (! all(which %in% 1:3))
             stop("if integer, argument 'which' should contain only 1, 2 and/or 3")
-        if (ncol(x) == 2 & 3 %in% which) stop("no grouping variable, only 2 indexes")
+        if (ncol(x) == 2 && 3 %in% which) stop("no grouping variable, only 2 indexes")
         which <- names(x)[which]
     }
     nindex <- names(x)
     gindex <- c("id", "time")
     if (ncol(x) == 3) gindex <- c(gindex, "group")
     if (any(! which %in% c(nindex, gindex))) stop("unknown variable")
-    if ("id"    %in% which) which[which == "id"]    <- names(x)[1]
-    if ("time"  %in% which) which[which == "time"]  <- names(x)[2]
-    if (ncol(x) == 3) if ("group" %in% which) which[which == "group"] <- names(x)[3]
+    if ("id"    %in% which) {
+      which[which == "id"]    <- names(x)[1L]
+      if("id" %in% names(x)[-1L]) warning("an index variable not being the invidiual index is called 'id'. Likely, any results are distorted.") 
+    }
+    if ("time"  %in% which) {
+      which[which == "time"]  <- names(x)[2L]
+      if("time" %in% names(x)[-2L]) warning("an index variable not being the time index is called 'time'. Likely, any results are distorted.") 
+    }
+    if (ncol(x) == 3) if ("group" %in% which) {
+      which[which == "group"] <- names(x)[3L]
+      if("group" %in% names(x)[-3L]) warning("an index variable not being the group index is called 'group'. Likely, any results are distorted.") 
+    }
+    
     result <- x[ , which]
     result
 }
 
 #' @rdname index.plm
 #' @export
-index.pdata.frame <- function(x, which = NULL, ...){
+index.pdata.frame <- function(x, which = NULL, ...) {
   anindex <- attr(x, "index")
   index(x = anindex, which = which)
 }
 
 #' @rdname index.plm
 #' @export
-index.pseries <- function(x, which = NULL, ...){
+index.pseries <- function(x, which = NULL, ...) {
   anindex <- attr(x, "index")
   index(x = anindex, which = which)
 }
   
 #' @rdname index.plm
 #' @export
-index.panelmodel <- function(x, which = NULL, ...){
+index.panelmodel <- function(x, which = NULL, ...) {
   anindex <- attr(x$model, "index")
   index(x = anindex, which = which)
 }
+
+
+is.index <- function(index) {
+  # not exported, helper function
+  # checks if the index is an index in the sense of package plm
+  res <- if (all(class(index) == c("pindex", "data.frame"))) TRUE else FALSE
+  return(res)
+}
+
+has.index <- function(object) {
+  # not exported, helper function
+  # checks if an object has an index in sense of package plm
+  # (esp. to distinguish from zoo::index() which always returns an index)
+  index <- attr(object, "index")
+  return(is.index(index))
+}
+
+checkNA.index <- function(index, which = "all", error = TRUE) {
+  # not exported, helper function
+  # check if any NA in indexes (all or specific dimension)
+  
+  feedback <- if(error) stop else warning
+  
+  if(which == "all") {
+    if(anyNA(index[[1L]])) feedback("NA in the individual index variable")
+    if(anyNA(index[[2L]])) feedback("NA in the time index variable")
+    if(ncol(index) == 3L) { if(anyNA(index[[3L]])) feedback("NA in the group index variable") }
+  }
+  if(which == 1L) {
+    if(anyNA(index[[1L]])) feedback("NA in the individual index variable")
+  }
+  if(which == 2L) {
+    if(anyNA(index[[2L]])) feedback("NA in the time index variable")
+  }
+  if(which == 3L) {
+    if(anyNA(index[[3L]])) feedback("NA in the group index variable")
+  }
+}
+
+# pos.index:
+# not exported, helper function
+#
+# determines column numbers of the index variables in a pdata.frame
+# returns named numeric of length 2 or 3 with column numbers of the index variables
+# (1: individual index, 2: time index, if available 3: group index), 
+# names are the names of the index variables
+#
+# returns c(NA, NA) / c(NA, NA, NA) if the index variables are not a column in the pdata.frame
+# (e.g., for pdata.frames created with drop.index = TRUE).
+# Cannot detect index variables if their columns names were changed after creation of the pdata.frame
+
+pos.index <- function(x, ...) {
+  index <- attr(x, "index")
+  index_names <- names(index)
+  index_pos <- match(index_names, names(x))
+  names(index_pos) <- index_names
+  return(index_pos)
+}
\ No newline at end of file
